@@ -22,10 +22,12 @@ package com.eteks.sweethome3d.swing;
 import java.awt.Color;
 import java.awt.Component;
 import java.awt.Cursor;
+import java.awt.Dimension;
 import java.awt.EventQueue;
 import java.awt.FileDialog;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.awt.image.BufferedImage;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.io.File;
@@ -38,10 +40,14 @@ import java.util.List;
 import java.util.Map;
 import java.util.Vector;
 import java.util.concurrent.Executor;
+import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
+import java.util.concurrent.atomic.AtomicReference;
 
+import javax.imageio.ImageIO;
 import javax.swing.AbstractAction;
 import javax.swing.Action;
+import javax.swing.BorderFactory;
 import javax.swing.JButton;
 import javax.swing.JComponent;
 import javax.swing.JDialog;
@@ -50,6 +56,8 @@ import javax.swing.JOptionPane;
 import javax.swing.JTree;
 import javax.swing.SwingUtilities;
 import javax.swing.UIManager;
+import javax.swing.event.AncestorEvent;
+import javax.swing.event.AncestorListener;
 import javax.swing.event.TreeExpansionEvent;
 import javax.swing.event.TreeExpansionListener;
 import javax.swing.event.TreeSelectionEvent;
@@ -215,7 +223,7 @@ public class FileContentManager implements ContentManager {
       new FileFilter() {
         @Override
         public boolean accept(File file) {
-          // Accept directories and .sh3d files
+          // Accept directories and .bmp files
           return file.isDirectory()
                  || file.getName().toLowerCase().endsWith(BMP_EXTENSION)
                  || file.getName().toLowerCase().endsWith(WBMP_EXTENSION);
@@ -316,6 +324,7 @@ public class FileContentManager implements ContentManager {
   
   private final UserPreferences           preferences;
   private final String                    sweetHome3DFileExtension;
+  private final String                    sweetHome3DFileExtension2;
   private final String                    languageLibraryFileExtension;
   private final String                    furnitureLibraryFileExtension;
   private final String                    texturesLibraryFileExtension;
@@ -327,6 +336,14 @@ public class FileContentManager implements ContentManager {
   public FileContentManager(final UserPreferences preferences) {  
     this.preferences = preferences;
     this.sweetHome3DFileExtension = preferences.getLocalizedString(FileContentManager.class, "homeExtension");
+    String homeExtension2;
+    try {
+      // Get optional second extension
+      homeExtension2 = preferences.getLocalizedString(FileContentManager.class, "homeExtension2");
+    } catch (IllegalArgumentException ex) {
+      homeExtension2 = null;
+    }
+    this.sweetHome3DFileExtension2 = homeExtension2;
     this.languageLibraryFileExtension = preferences.getLocalizedString(FileContentManager.class, "languageLibraryExtension");
     this.furnitureLibraryFileExtension = preferences.getLocalizedString(FileContentManager.class, "furnitureLibraryExtension");
     this.texturesLibraryFileExtension = preferences.getLocalizedString(FileContentManager.class, "texturesLibraryExtension");
@@ -348,9 +365,11 @@ public class FileContentManager implements ContentManager {
         new FileFilter() {
           @Override
           public boolean accept(File file) {
-            // Accept directories and .sh3d files
+            // Accept directories, .sh3d and .sh3x files
             return file.isDirectory()
-                || file.getName().toLowerCase().endsWith(sweetHome3DFileExtension);
+                || file.getName().toLowerCase().endsWith(sweetHome3DFileExtension)
+                || (sweetHome3DFileExtension2 != null 
+                     && file.getName().toLowerCase().endsWith(sweetHome3DFileExtension2));
           }
           
           @Override
@@ -436,11 +455,14 @@ public class FileContentManager implements ContentManager {
 
     // Fill file default extension map
     this.fileExtensions = new HashMap<ContentType, String []>();
-    this.fileExtensions.put(ContentType.SWEET_HOME_3D,     new String [] {sweetHome3DFileExtension});
-    this.fileExtensions.put(ContentType.LANGUAGE_LIBRARY,  new String [] {languageLibraryFileExtension});
-    this.fileExtensions.put(ContentType.FURNITURE_LIBRARY, new String [] {furnitureLibraryFileExtension});
-    this.fileExtensions.put(ContentType.TEXTURES_LIBRARY,  new String [] {texturesLibraryFileExtension});
-    this.fileExtensions.put(ContentType.PLUGIN,            new String [] {pluginFileExtension});
+    String [] sweetHome3DFileExtensions = this.sweetHome3DFileExtension2 != null
+        ? new String [] {this.sweetHome3DFileExtension, this.sweetHome3DFileExtension2}
+        : new String [] {this.sweetHome3DFileExtension};
+    this.fileExtensions.put(ContentType.SWEET_HOME_3D,     sweetHome3DFileExtensions);
+    this.fileExtensions.put(ContentType.LANGUAGE_LIBRARY,  new String [] {this.languageLibraryFileExtension});
+    this.fileExtensions.put(ContentType.FURNITURE_LIBRARY, new String [] {this.furnitureLibraryFileExtension});
+    this.fileExtensions.put(ContentType.TEXTURES_LIBRARY,  new String [] {this.texturesLibraryFileExtension});
+    this.fileExtensions.put(ContentType.PLUGIN,            new String [] {this.pluginFileExtension});
     this.fileExtensions.put(ContentType.PNG,               new String [] {PNG_EXTENSION});
     this.fileExtensions.put(ContentType.JPEG,              new String [] {JPEG_EXTENSION});
     this.fileExtensions.put(ContentType.MOV,               new String [] {MOV_EXTENSION});
@@ -473,6 +495,10 @@ public class FileContentManager implements ContentManager {
                                     ContentType contentType) {
     switch (contentType) {
       case SWEET_HOME_3D :
+      case FURNITURE_LIBRARY :
+      case TEXTURES_LIBRARY :
+      case LANGUAGE_LIBRARY :
+      case PLUGIN :
         return new File(contentPath).getName();
       default :
         String fileName = new File(contentPath).getName();
@@ -527,8 +553,9 @@ public class FileContentManager implements ContentManager {
    */
   public boolean isAcceptable(String contentPath, 
                               ContentType contentType) {
+    File file = new File(contentPath);
     for (FileFilter filter : getFileFilter(contentType)) {
-      if (filter.accept(new File(contentPath))) {
+      if (filter.accept(file)) {
         return true;
       }
     }
@@ -638,7 +665,7 @@ public class FileContentManager implements ContentManager {
         }
       });
 
-      // Update directory
+    // Update directory
     File directory = getLastDirectory(contentType);
     if (directory != null && directory.exists()) {
       if (isDirectory(contentType)) {
@@ -711,12 +738,93 @@ public class FileContentManager implements ContentManager {
                                  ContentType   contentType,
                                  String        path,
                                  boolean       save) {
+    if (OperatingSystem.isWindows()
+        && UIManager.getLookAndFeel().getClass().getName().equals(UIManager.getSystemLookAndFeelClassName())
+        // Workaround for bug http://bugs.java.com/bugdatabase/view_bug.do?bug_id=8179014 under Windows 10 Creator update
+        && !OperatingSystem.isJavaVersionGreaterOrEqual("1.8.0_141")
+        // Testing Windows 10 version changed fixed in Java 8u60 until which "os.version" was equal to "6.3"
+        && (!OperatingSystem.isJavaVersionGreaterOrEqual("1.8.0_60") && OperatingSystem.compareVersions(System.getProperty("os.version"), "6.3") >= 0
+             || OperatingSystem.isJavaVersionGreaterOrEqual("1.8.0_60") && OperatingSystem.compareVersions(System.getProperty("os.version"), "10.0") >= 0)) {
+      UIManager.put("FileChooser.useSystemExtensionHiding", false);
+    }
+  
     final JFileChooser fileChooser;
     if (isDirectory(contentType)) {
       fileChooser = new DirectoryChooser(this.preferences);
     } else {
       fileChooser = new JFileChooser();
+      if (!save
+          && contentType == ContentType.IMAGE) {
+        // Add a preview component when the file chooser is used to select an image
+        final ScaledImageComponent previewLabel = new ScaledImageComponent();
+        final ExecutorService previewImageLoader = Executors.newSingleThreadExecutor();
+        final AtomicReference<File> selectedImageFile = new AtomicReference<File>();
+        fileChooser.addPropertyChangeListener(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY, 
+            new PropertyChangeListener() {
+              public void propertyChange(PropertyChangeEvent ev) {
+                final File file = (File)ev.getNewValue();
+                previewLabel.setImage(null);
+                if (file != null
+                    && !file.isDirectory()
+                    && isAcceptable(file.getPath(), ContentType.IMAGE)) {
+                  fileChooser.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
+                  selectedImageFile.set(file);
+                  previewImageLoader.execute(new Runnable() {
+                      public void run() {
+                        BufferedImage image = null;
+                        try {
+                          // Check the file is the selected image file  
+                          if (selectedImageFile.get() == file) {
+                            image = ImageIO.read(file);
+                          }
+                        } catch (IOException ex) {
+                          // Image couldn't be loaded
+                        } finally {
+                          final BufferedImage previewedImage = image;
+                          EventQueue.invokeLater(new Runnable() {
+                              public void run() {
+                                if (selectedImageFile.get() == file) {
+                                  previewLabel.setImage(previewedImage);
+                                  fileChooser.setCursor(Cursor.getDefaultCursor());
+                                }
+                              }
+                            });
+                        }
+                      }
+                    });
+                } else {
+                  selectedImageFile.set(null);
+                  fileChooser.setCursor(Cursor.getDefaultCursor());
+                }
+              }
+            });
+        fileChooser.addPropertyChangeListener(JFileChooser.DIRECTORY_CHANGED_PROPERTY, 
+            new PropertyChangeListener() {
+              public void propertyChange(PropertyChangeEvent ev) {
+                previewLabel.setImage(null);
+                selectedImageFile.set(null);
+                fileChooser.setCursor(Cursor.getDefaultCursor());
+              }
+            });
+        previewLabel.addAncestorListener(new AncestorListener() {
+            public void ancestorRemoved(AncestorEvent event) {
+              previewImageLoader.shutdownNow(); 
+            }
+            
+            public void ancestorAdded(AncestorEvent event) {
+            }
+  
+            public void ancestorMoved(AncestorEvent event) {
+            }
+          });
+        previewLabel.setBorder(BorderFactory.createCompoundBorder(
+            BorderFactory.createEmptyBorder(0, OperatingSystem.isMacOSX() ? 0 : 5, 0, OperatingSystem.isMacOSX() ? 5 : 0), 
+            BorderFactory.createLineBorder(Color.GRAY)));
+        previewLabel.setPreferredSize(new Dimension(128 + 12, 128 + 2));   
+        fileChooser.setAccessory(previewLabel);
+      }
     }
+    
     if (dialogTitle == null) {
       dialogTitle = getFileDialogTitle(save);
     }
